/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/* $Rev: 489263 $ $Date: 2006-12-21 05:21:32 +0000 (Thu, 21 Dec 2006) $ */

#ifndef tuscany_sca_extension_servicebindingextension_h
#define tuscany_sca_extension_servicebindingextension_h

#include <string>

#include "commonj/sdo/SDO.h"

#include "tuscany/sca/export.h"
#include "tuscany/sca/model/Component.h"
#include "tuscany/sca/model/Service.h"
#include "tuscany/sca/core/ServiceWrapper.h"
#include "tuscany/sca/model/Composite.h"
#include "tuscany/sca/model/ServiceBinding.h"


namespace tuscany
{
    namespace sca
    {
        class SCA_API ServiceBindingExtension 
        {
        public:
            /**
            * Default constructor
            */
            ServiceBindingExtension();            
            
            /**
            * Destructor
            */
            virtual ~ServiceBindingExtension();            
                      
            /**
            * return the name of the extension (e.g. "cpp" "php")
            */
            virtual const std::string& getExtensionName() = 0;
                      
            /**
            * return the QName of schema elemant for this binding extension
            * (e.g. "http://www.osoa.org/xmlns/sca/1.0#binding.ws")
            */
            virtual const std::string& getExtensionTypeQName() = 0;

            /**
             * Get a reference binding from a DataObject representing
             * an SCDL binding element
             */ 
            virtual tuscany::sca::model::ServiceBinding* getServiceBinding(
                tuscany::sca::model::Composite* composite,
                tuscany::sca::model::Service* service,
                commonj::sdo::DataObjectPtr scdlBinding) = 0;

         };

        
    } // End namespace sca
} // End namespace tuscany

#endif // tuscany_sca_extension_servicebindingextension_h

