/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/* $Rev: 489263 $ $Date: 2006-12-21 05:21:32 +0000 (Thu, 21 Dec 2006) $ */

#ifndef tuscany_sca_model_wire_h
#define tuscany_sca_model_wire_h

#include <string>

namespace tuscany
{
    namespace sca
    {
        namespace model
        {
            /**
             * Information about a wire in the model.
             */
            class Wire
            {
            public: 
            
                /**
                 * Constructor.
                 * @param source The source of the wire. Either the component and 
                 * reference name (optional) or an entry point.
                 * @param target The target of the wire. Either a component and service
                 * service name (optional) or an external sevice.
                 */
                SCA_API Wire(const std::string& source, const std::string& target);

                /**
                 * Destructor.
                 */
                SCA_API virtual ~Wire();

                /**
                 * Get the component name defined by the source of the wire.
                 * @return The component name which is the source of the wire.
                 */
                SCA_API const std::string& getSourceComponent() const { return sourceComponent; }

                /**
                 * Get the reference name defined by the source of the wire.
                 * @return The reference name which is the source of the wire.
                 */
                SCA_API const std::string& getSourceReference() const { return sourceReference; }

                /**
                 * Get the target uri defined by the target of the wire.
                 * @return The target uri which is the source of the wire.
                 */
                SCA_API const std::string& getTarget() { return target; }
                
             private:
                /**
                 * The source component of the wire.
                 */
                std::string sourceComponent;

                /**
                 * The source reference of the wire.
                 */
                std::string sourceReference;

                /**
                 * The target uri of the wire.
                 */
                 std::string target;
            };
            
        } // End namespace model
    } // End namespace sca
} // End namespace tuscany

#endif // tuscany_sca_model_wire_h

