/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/* $Rev: 489263 $ $Date: 2006-12-21 05:21:32 +0000 (Thu, 21 Dec 2006) $ */

#include "tuscany/sca/util/Logging.h"
#include "tuscany/sca/cpp/model/CPPImplementation.h"
#include "tuscany/sca/cpp/model/CPPServiceBinding.h"
#include "tuscany/sca/cpp/model/CPPReferenceBinding.h"
#include "tuscany/sca/model/Component.h"
#include "tuscany/sca/model/Composite.h"
#include "tuscany/sca/model/Service.h"
#include "tuscany/sca/model/Reference.h"
#include "tuscany/sca/util/Utils.h"

using namespace std;
using namespace tuscany::sca::model;

namespace tuscany
{
    namespace sca
    {

        namespace cpp
        {

            // Constructor
            CPPImplementation::CPPImplementation(Composite* composite, const string& library, const string& header,
                    const string&headerPath, const string& headerStub, const string& className, Scope scope)
                : ComponentType(composite, headerPath + headerStub),
                    library(library), header(header), headerPath(headerPath),
                    headerStub(headerStub), className(className), scope(scope)
            {
            }

            CPPImplementation::~CPPImplementation()
            {
            }
            
            void CPPImplementation::initializeComponent(Component* component)
            {
                ComponentType::initializeComponent(component);
                
                // Create CPP bindings for all the services
                const Component::SERVICE_MAP& services = component->getServices();
                Component::SERVICE_MAP::const_iterator iter = services.begin();
                for (unsigned int i=0; i< services.size(); i++)
                {
                    Service *service = iter->second;
                    CPPServiceBinding* binding = new CPPServiceBinding(service);
                    service->setBinding(binding);
                    iter++;
                }
                
                // Create CPP bindings for all the references
                const Component::REFERENCE_MAP& references = component->getReferences();
                Component::REFERENCE_MAP::const_iterator refiter = references.begin();
                for (unsigned int ri=0; ri< references.size(); ri++)
                {
                    Reference *reference = refiter->second;
                    CPPReferenceBinding* binding = new CPPReferenceBinding(reference);
                    reference->setBinding(binding);
                    refiter++;
                }
            }
            
        } // End namespace cpp
    } // End namespace sca
} // End namespace tuscany
